@@ -19,6 +19,8 @@
     Home: https://github.com/gorhill/uBlock
 */
 
+'use strict';
+
 /*******************************************************************************
 
 A PageRequestStore object is used to store net requests in two ways:
@@ -33,8 +35,6 @@ To create a log of net requests
 
 µBlock.PageStore = (function() {
 
-'use strict';
-
 /******************************************************************************/
 
 var µb = µBlock;
@@ -58,13 +58,13 @@ NetFilteringResultCacheEntry.prototype.init = function(result, type) {
     this.result = result;
     this.type = type;
     this.time = Date.now();
+    return this;
 };
 
 /******************************************************************************/
 
 NetFilteringResultCacheEntry.prototype.dispose = function() {
-    this.result = '';
-    this.type = '';
+    this.result = this.type = '';
     if ( netFilteringResultCacheEntryJunkyard.length < netFilteringResultCacheEntryJunkyardMax ) {
         netFilteringResultCacheEntryJunkyard.push(this);
     }
@@ -73,13 +73,10 @@ NetFilteringResultCacheEntry.prototype.dispose = function() {
 /******************************************************************************/
 
 NetFilteringResultCacheEntry.factory = function(result, type) {
-    var entry = netFilteringResultCacheEntryJunkyard.pop();
-    if ( entry === undefined ) {
-        entry = new NetFilteringResultCacheEntry(result, type);
-    } else {
-        entry.init(result, type);
+    if ( netFilteringResultCacheEntryJunkyard.length ) {
+        return netFilteringResultCacheEntryJunkyard.pop().init(result, type);
     }
-    return entry;
+    return new NetFilteringResultCacheEntry(result, type);
 };
 
 /******************************************************************************/
@@ -110,7 +107,7 @@ NetFilteringResultCache.factory = function() {
 /******************************************************************************/
 
 NetFilteringResultCache.prototype.init = function() {
-    this.urls = {};
+    this.urls = Object.create(null);
     this.count = 0;
     this.shelfLife = 15 * 1000;
     this.timer = null;
@@ -131,16 +128,17 @@ NetFilteringResultCache.prototype.dispose = function() {
 /******************************************************************************/
 
 NetFilteringResultCache.prototype.add = function(context, result) {
-    var url = context.requestURL;
-    var type = context.requestType;
-    var entry = this.urls[url];
+    var url = context.requestURL,
+        type = context.requestType,
+        key = type + ' ' + url,
+        entry = this.urls[key];
     if ( entry !== undefined ) {
         entry.result = result;
         entry.type = type;
         entry.time = Date.now();
         return;
     }
-    this.urls[url] = NetFilteringResultCacheEntry.factory(result, type);
+    this.urls[key] = NetFilteringResultCacheEntry.factory(result, type);
     if ( this.count === 0 ) {
         this.pruneAsync();
     }
@@ -151,12 +149,9 @@ NetFilteringResultCache.prototype.add = function(context, result) {
 
 NetFilteringResultCache.prototype.empty = function() {
     for ( var key in this.urls ) {
-        if ( this.urls.hasOwnProperty(key) === false ) {
-            continue;
-        }
         this.urls[key].dispose();
     }
-    this.urls = {};
+    this.urls = Object.create(null);
     this.count = 0;
     if ( this.timer !== null ) {
         clearTimeout(this.timer);
@@ -315,26 +310,42 @@ PageStore.prototype.init = function(tabId) {
     this.largeMediaTimer = null;
     this.netFilteringCache = NetFilteringResultCache.factory();
 
-    // Support `elemhide` filter option. Called at this point so the required
-    // context is all setup at this point.
-    this.skipCosmeticFiltering = µb.staticNetFilteringEngine.matchStringExactType(
-        this.createContextFromPage(),
-        tabContext.normalURL,
-        'elemhide'
-    ) === false;
-    if ( this.skipCosmeticFiltering && µb.logger.isEnabled() ) {
-        // https://github.com/gorhill/uBlock/issues/370
-        // Log using `cosmetic-filtering`, not `elemhide`.
+    this.noCosmeticFiltering = µb.hnSwitches.evaluateZ('no-cosmetic-filtering', tabContext.rootHostname) === true;
+    if ( µb.logger.isEnabled() && this.noCosmeticFiltering ) {
         µb.logger.writeOne(
             tabId,
-            'net',
-            µb.staticNetFilteringEngine.toResultString(true),
-            'elemhide',
+            'cosmetic',
+            µb.hnSwitches.toResultString(),
+            'dom',
             tabContext.rawURL,
             this.tabHostname,
             this.tabHostname
         );
     }
+
+    // Support `generichide` filter option.
+    this.noGenericCosmeticFiltering = this.noCosmeticFiltering;
+    if ( this.noGenericCosmeticFiltering !== true ) {
+        this.noGenericCosmeticFiltering = µb.staticNetFilteringEngine.matchStringExactType(
+            this.createContextFromPage(),
+            tabContext.normalURL,
+            'elemhide'
+        ) === false;
+        if ( µb.logger.isEnabled() && this.noGenericCosmeticFiltering ) {
+            // https://github.com/gorhill/uBlock/issues/370
+            // Log using `cosmetic-filtering`, not `elemhide`.
+            µb.logger.writeOne(
+                tabId,
+                'net',
+                µb.staticNetFilteringEngine.toResultString(true),
+                'elemhide',
+                tabContext.rawURL,
+                this.tabHostname,
+                this.tabHostname
+            );
+        }
+    }
+
     return this;
 };
 
@@ -428,14 +439,14 @@ PageStore.prototype.setFrame = function(frameId, frameURL) {
 /******************************************************************************/
 
 PageStore.prototype.createContextFromPage = function() {
-    var context = new µb.tabContextManager.createContext(this.tabId);
+    var context = µb.tabContextManager.createContext(this.tabId);
     context.pageHostname = context.rootHostname;
     context.pageDomain = context.rootDomain;
     return context;
 };
 
 PageStore.prototype.createContextFromFrameId = function(frameId) {
-    var context = new µb.tabContextManager.createContext(this.tabId);
+    var context = µb.tabContextManager.createContext(this.tabId);
     var frameStore = this.frames[frameId];
     if ( frameStore ) {
         context.pageHostname = frameStore.pageHostname;
@@ -448,7 +459,7 @@ PageStore.prototype.createContextFromFrameId = function(frameId) {
 };
 
 PageStore.prototype.createContextFromFrameHostname = function(frameHostname) {
-    var context = new µb.tabContextManager.createContext(this.tabId);
+    var context = µb.tabContextManager.createContext(this.tabId);
     context.pageHostname = frameHostname;
     context.pageDomain = µb.URI.domainFromHostname(frameHostname) || frameHostname;
     return context;
@@ -463,16 +474,14 @@ PageStore.prototype.getNetFilteringSwitch = function() {
 /******************************************************************************/
 
 PageStore.prototype.getSpecificCosmeticFilteringSwitch = function() {
-    var tabContext = µb.tabContextManager.lookup(this.tabId);
-    return tabContext !== null &&
-           µb.hnSwitches.evaluateZ('no-cosmetic-filtering', tabContext.rootHostname) !== true;
+    return this.noCosmeticFiltering !== true;
 };
 
 /******************************************************************************/
 
 PageStore.prototype.getGenericCosmeticFilteringSwitch = function() {
-    return this.skipCosmeticFiltering !== true &&
-           this.getSpecificCosmeticFilteringSwitch();
+    return this.noGenericCosmeticFiltering !== true &&
+           this.noCosmeticFiltering !== true;
 };
 
 /******************************************************************************/
