@@ -21,12 +21,12 @@
 
 /* global punycode, uDom */
 
+'use strict';
+
 /******************************************************************************/
 
 (function() {
 
-'use strict';
-
 /******************************************************************************/
 
 // Ensure the popup is properly sized as soon as possible. It is assume the DOM
@@ -507,23 +507,24 @@ var renderPopup = function() {
 /******************************************************************************/
 
 var renderPopupLazy = function() {
-    var onDataReady = function(data) {
-        if ( !data ) { return; }
-        var v = data.hiddenElementCount || '';
+    messaging.send('popupPanel', { what: 'getPopupLazyData', tabId: popupData.tabId });
+};
+
+var onPopupMessage = function(data) {
+    if ( !data ) { return; }
+    if ( data.tabId !== popupData.tabId ) { return; }
+
+    switch ( data.what ) {
+    case 'cosmeticallyFilteredElementCountChanged':
+        var v = data.count || '';
         uDom.nodeFromSelector('#no-cosmetic-filtering > span.badge')
             .textContent = typeof v === 'number' ? v.toLocaleString() : v;
-    };
-
-    messaging.send(
-        'popupPanel',
-        {
-            what: 'getPopupDataLazy',
-            tabId: popupData.tabId
-        },
-        onDataReady
-    );
+        break;
+    }
 };
 
+messaging.addChannelListener('popup', onPopupMessage);
+
 /******************************************************************************/
 
 var toggleNetFilteringSwitch = function(ev) {
@@ -570,8 +571,6 @@ var gotoURL = function(ev) {
 
     ev.preventDefault();
 
-    var rel = this.getAttribute('rel') || '';
-
     messaging.send(
         'popupPanel',
         {
@@ -580,7 +579,7 @@ var gotoURL = function(ev) {
                 url: this.getAttribute('href'),
                 select: true,
                 index: -1,
-                popup: rel === 'popup' && ev.shiftKey
+                shiftKey: ev.shiftKey
             }
         }
     );
