@@ -1,7 +1,7 @@
 /*******************************************************************************
 
-    uBlock - a browser extension to block requests.
-    Copyright (C) 2014-2015 Raymond Hill
+    uBlock Origin - a browser extension to block requests.
+    Copyright (C) 2014-2016 Raymond Hill
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,12 +22,12 @@
 /* jshint bitwise: false */
 /* global punycode */
 
+'use strict';
+
 /******************************************************************************/
 
 µBlock.staticNetFilteringEngine = (function(){
 
-'use strict';
-
 /******************************************************************************/
 
 var µb = µBlock;
@@ -55,6 +55,7 @@ var ThirdParty = 2 << 2;
 
 var AnyType = 0 << 4;
 var typeNameToTypeValue = {
+           'no_type':  0 << 4,
         'stylesheet':  1 << 4,
              'image':  2 << 4,
             'object':  3 << 4,
@@ -105,8 +106,6 @@ var AllowAnyTypeAnyParty = AllowAction | AnyType | AnyParty;
 var AllowAnyType = AllowAction | AnyType;
 var AllowAnyParty = AllowAction | AnyParty;
 
-var reURLPostHostnameAnchors = /[\/?#]/;
-
 // ABP filters: https://adblockplus.org/en/filters
 // regex tester: http://regex101.com/
 
@@ -172,8 +171,8 @@ var atoi = function(s) {
     return cachedParseInt(s, 10);
 };
 
+// Be sure to not confuse 'example.com' with 'anotherexample.com'
 var isFirstParty = function(domain, hostname) {
-    // Be sure to not confuse 'example.com' with 'anotherexample.com'
     return hostname.endsWith(domain) &&
           (hostname.length === domain.length ||
            hostname.charAt(hostname.length - domain.length - 1) === '.');
@@ -206,11 +205,17 @@ var strToRegex = function(s, anchor, flags) {
     if ( s === '*' ) {
         return alwaysTruePseudoRegex;
     }
-
+    var anchorToHnStart;
+    if ( s.startsWith('||') ) {
+        s = s.slice(2);
+        anchorToHnStart = s.charCodeAt(0) === 0x2A;
+    }
     // https://www.loggly.com/blog/five-invaluable-techniques-to-improve-regex-performance/
     // https://developer.mozilla.org/en/docs/Web/JavaScript/Guide/Regular_Expressions
+    // Also: remove leading/trailing wildcards -- there is no point.
     var reStr = s.replace(/[.+?${}()|[\]\\]/g, '\\$&')
                  .replace(/\^/g, '(?:[^%.0-9a-z_-]|$)')
+                 .replace(/^\*|\*$/g, '')
                  .replace(/\*/g, '[^ ]*?');
 
     if ( anchor < 0 ) {
@@ -218,7 +223,9 @@ var strToRegex = function(s, anchor, flags) {
     } else if ( anchor > 0 ) {
         reStr += '$';
     }
-
+    if ( anchorToHnStart ) {
+        reStr = '[0-9a-z.-]*?' + reStr;
+    }
     //console.debug('µBlock.staticNetFilteringEngine: created RegExp("%s")', reStr);
     return new RegExp(reStr, flags);
 };
@@ -227,6 +234,26 @@ var toHex = function(n) {
     return n.toString(16);
 };
 
+// First character of match must be within the hostname part of the url.
+var isHnAnchored = function(url, matchStart) {
+    var hnStart = url.indexOf('://');
+    if ( hnStart === -1 ) {
+        return false;
+    }
+    hnStart += 3;
+    if ( matchStart <= hnStart ) {
+        return true;
+    }
+    if ( reURLPostHostnameAnchors.test(url.slice(hnStart, matchStart)) ) {
+        return false;
+    }
+    // https://github.com/gorhill/uBlock/issues/1929
+    // Match only hostname label boundaries.
+    return url.charCodeAt(matchStart - 1) === 0x2E;
+};
+
+var reURLPostHostnameAnchors = /[\/?#]/;
+
 /******************************************************************************/
 
 // Hostname test helpers: the optimal test function is picked
@@ -697,13 +724,8 @@ var FilterPlainHnAnchored = function(s) {
 };
 
 FilterPlainHnAnchored.prototype.match = function(url, tokenBeg) {
-    if ( url.startsWith(this.s, tokenBeg) === false ) {
-        return false;
-    }
-    // Valid only if hostname-valid characters to the left of token
-    var pos = url.indexOf('://');
-    return pos !== -1 &&
-           reURLPostHostnameAnchors.test(url.slice(pos + 3, tokenBeg)) === false;
+    return url.startsWith(this.s, tokenBeg) &&
+           isHnAnchored(url, tokenBeg);
 };
 
 FilterPlainHnAnchored.fid =
@@ -737,16 +759,9 @@ var FilterPlainHnAnchoredHostname = function(s, domainOpt) {
 };
 
 FilterPlainHnAnchoredHostname.prototype.match = function(url, tokenBeg) {
-    if (
-        url.startsWith(this.s, tokenBeg) === false ||
-        this.hostnameTest(this) === false
-    ) {
-        return false;
-    }
-    // Valid only if hostname-valid characters to the left of token
-    var pos = url.indexOf('://');
-    return pos !== -1 &&
-           reURLPostHostnameAnchors.test(url.slice(pos + 3, tokenBeg)) === false;
+    return url.startsWith(this.s, tokenBeg) &&
+           this.hostnameTest(this) &&
+           isHnAnchored(url, tokenBeg);
 };
 
 FilterPlainHnAnchoredHostname.fid =
@@ -850,18 +865,10 @@ var FilterGenericHnAnchored = function(s) {
 
 FilterGenericHnAnchored.prototype.match = function(url) {
     if ( this.re === null ) {
-        this.re = strToRegex(this.s, 0);
+        this.re = strToRegex('||' + this.s, 0);
     }
-    // Quick test first
-    if ( this.re.test(url) === false ) {
-        return false;
-    }
-    // Valid only if begininning of match is within the hostname
-    // part of the url
-    var match = this.re.exec(url);
-    var pos = url.indexOf('://');
-    return pos !== -1 &&
-           reURLPostHostnameAnchors.test(url.slice(pos + 3, match.index)) === false;
+    var matchStart = url.search(this.re);
+    return matchStart !== -1 && isHnAnchored(url, matchStart);
 };
 
 FilterGenericHnAnchored.fid =
@@ -919,7 +926,7 @@ FilterGenericHnAnchoredHostname.fromSelfie = function(s) {
 // Regex-based filters
 
 var FilterRegex = function(s) {
-    this.re = new RegExp(s);
+    this.re = new RegExp(s, 'i');
 };
 
 FilterRegex.prototype.match = function(url) {
@@ -946,7 +953,7 @@ FilterRegex.fromSelfie = function(s) {
 /******************************************************************************/
 
 var FilterRegexHostname = function(s, domainOpt) {
-    this.re = new RegExp(s);
+    this.re = new RegExp(s, 'i');
     this.domainOpt = domainOpt;
     this.hostnameTest = hostnameTestPicker(this);
 };
@@ -980,166 +987,24 @@ FilterRegexHostname.fromSelfie = function(s) {
 
 // Dictionary of hostnames
 //
-// FilterHostnameDict is the main reason why uBlock is not equipped to keep
-// track of which filter comes from which list, and also why it's not equipped
-// to be able to disable a specific filter -- other than through using a
-// counter-filter.
-//
-// On the other hand it is also *one* of the reason uBlock's memory and CPU
-// footprint is smaller. Compacting huge list of hostnames into single strings
-// saves a lot of memory compared to having one dictionary entry per hostname.
-
 var FilterHostnameDict = function() {
     this.h = ''; // short-lived register
-    this.dict = {};
-    this.count = 0;
-};
-
-// Somewhat arbitrary: I need to come up with hard data to know at which
-// point binary search is better than indexOf.
-//
-// http://jsperf.com/string-indexof-vs-binary-search
-// Tuning above performance benchmark, it appears 250 is roughly a good value
-// for both Chromium/Firefox.
-// Example of benchmark values: '------30', '-----100', etc. -- the
-// needle string must always be 8-character long.
-
-FilterHostnameDict.prototype.cutoff = 250;
-
-// Probably not needed under normal circumstances.
-
-FilterHostnameDict.prototype.meltBucket = function(len, bucket) {
-    var map = {};
-    if ( bucket.startsWith(' ') ) {
-        bucket.trim().split(' ').map(function(k) {
-            map[k] = true;
-        });
-    } else {
-        var offset = 0;
-        while ( offset < bucket.length ) {
-            map[bucket.substr(offset, len)] = true;
-            offset += len;
-        }
-    }
-    return map;
-};
-
-FilterHostnameDict.prototype.freezeBucket = function(bucket) {
-    var hostnames = Object.keys(bucket);
-    if ( hostnames[0].length * hostnames.length < this.cutoff ) {
-        return ' ' + hostnames.join(' ') + ' ';
-    }
-    return hostnames.sort().join('');
-};
-
-// How the key is derived dictates the number and size of buckets:
-// - more bits = more buckets = higher memory footprint
-// - less bits = less buckets = lower memory footprint
-// - binary search mitigates very well the fact that some buckets may grow
-//   large when fewer bits are used (or when a large number of items are
-//   stored). Binary search also mitigate to the point of non-issue the
-//   CPU footprint requirement with large buckets, as far as reference
-//   benchmark shows.
-//
-// A hash key capable of better spread while being as fast would be
-// just great.
-
-FilterHostnameDict.prototype.makeKey = function(hn) {
-    var len = hn.length;
-    if ( len > 255 ) {
-        len = 255;
-    }
-    var i8 = len >>> 3;
-    var i4 = len >>> 2;
-    var i2 = len >>> 1;
-
-    // http://jsperf.com/makekey-concat-vs-join/3
-
-    // Be sure the msb is not set, this will guarantee a valid unicode
-    // character (because 0xD800-0xDFFF).
-    return String.fromCharCode(
-        (hn.charCodeAt(      i8) & 0x01) << 14 |
-//        (hn.charCodeAt(   i4   ) & 0x01) << 13 |
-        (hn.charCodeAt(   i4+i8) & 0x01) << 12 |
-        (hn.charCodeAt(i2      ) & 0x01) << 11 |
-        (hn.charCodeAt(i2   +i8) & 0x01) << 10 |
-//        (hn.charCodeAt(i2+i4   ) & 0x01) <<  9 |
-        (hn.charCodeAt(i2+i4+i8) & 0x01) <<  8 ,
-        len
-    );
+    this.dict = new Set();
 };
 
 FilterHostnameDict.prototype.add = function(hn) {
-    var key = this.makeKey(hn);
-    var bucket = this.dict[key];
-    if ( bucket === undefined ) {
-        bucket = this.dict[key] = {};
-        bucket[hn] = true;
-        this.count += 1;
-        return true;
-    }
-    if ( typeof bucket === 'string' ) {
-        bucket = this.dict[key] = this.meltBucket(hn.length, bucket);
-    }
-    if ( bucket.hasOwnProperty(hn) ) {
+    if ( this.dict.has(hn) ) {
         return false;
     }
-    bucket[hn] = true;
-    this.count += 1;
+    this.dict.add(hn);
     return true;
 };
 
-FilterHostnameDict.prototype.freeze = function() {
-    var buckets = this.dict;
-    var bucket;
-    for ( var key in buckets ) {
-        bucket = buckets[key];
-        if ( typeof bucket === 'object' ) {
-            buckets[key] = this.freezeBucket(bucket);
-        }
-    }
-};
-
-FilterHostnameDict.prototype.matchesExactly = function(hn) {
-    // TODO: Handle IP address
-
-    var key = this.makeKey(hn);
-    var bucket = this.dict[key];
-    if ( bucket === undefined ) {
-        return false;
-    }
-    if ( typeof bucket === 'object' ) {
-        bucket = this.dict[key] = this.freezeBucket(bucket);
-    }
-    if ( bucket.startsWith(' ') ) {
-        return bucket.indexOf(' ' + hn + ' ') !== -1;
-    }
-    // binary search
-    var len = hn.length;
-    var left = 0;
-    // http://jsperf.com/or-vs-floor/17
-    var right = (bucket.length / len + 0.5) | 0;
-    var i, needle;
-    while ( left < right ) {
-        i = left + right >> 1;
-        needle = bucket.substr( len * i, len );
-        if ( hn < needle ) {
-            right = i;
-        } else if ( hn > needle ) {
-            left = i + 1;
-        } else {
-            return true;
-        }
-    }
-    return false;
-};
-
 FilterHostnameDict.prototype.match = function() {
     // TODO: mind IP addresses
-
     var pos,
         hostname = requestHostnameRegister;
-    while ( this.matchesExactly(hostname) === false ) {
+    while ( this.dict.has(hostname) === false ) {
         pos = hostname.indexOf('.');
         if ( pos === -1 ) {
             this.h = '';
@@ -1160,17 +1025,12 @@ FilterHostnameDict.prototype.rtCompile = function() {
 };
 
 FilterHostnameDict.prototype.toSelfie = function() {
-    return JSON.stringify({
-        count: this.count,
-        dict: this.dict
-    });
+    return JSON.stringify(µb.setToArray(this.dict));
 };
 
 FilterHostnameDict.fromSelfie = function(s) {
     var f = new FilterHostnameDict();
-    var o = JSON.parse(s);
-    f.count = o.count;
-    f.dict = o.dict;
+    f.dict = µb.setFromArray(JSON.parse(s));
     return f;
 };
 
@@ -1287,6 +1147,7 @@ FilterBucket.fromSelfie = function() {
 /******************************************************************************/
 
 var FilterParser = function() {
+    this.cantWebsocket = vAPI.cantWebsocket;
     this.reHostnameRule1 = /^[0-9a-z][0-9a-z.-]*[0-9a-z]$/i;
     this.reHostnameRule2 = /^\**[0-9a-z][0-9a-z.-]*[0-9a-z]\^?$/i;
     this.reCleanupHostnameRule2 = /^\**|\^$/g;
@@ -1296,6 +1157,7 @@ var FilterParser = function() {
     this.reHasUppercase = /[A-Z]/;
     this.reIsolateHostname = /^(\*?\.)?([^\x00-\x24\x26-\x2C\x2F\x3A-\x5E\x60\x7B-\x7F]+)(.*)/;
     this.reHasUnicode = /[^\x00-\x7F]/;
+    this.reWebsocketAny = /^ws[s*]?(?::\/?\/?)?\*?$/;
     this.domainOpt = '';
     this.reset();
 };
@@ -1347,11 +1209,17 @@ FilterParser.prototype.reset = function() {
 
 /******************************************************************************/
 
+FilterParser.prototype.bitFromType = function(type) {
+    return 1 << ((typeNameToTypeValue[type] >>> 4) - 1);
+};
+
+/******************************************************************************/
+
 // https://github.com/chrisaljoudi/uBlock/issues/589
 // Be ready to handle multiple negated types
 
 FilterParser.prototype.parseOptType = function(raw, not) {
-    var typeBit = 1 << ((typeNameToTypeValue[this.toNormalizedType[raw]] >>> 4) - 1);
+    var typeBit = this.bitFromType(this.toNormalizedType[raw]);
 
     if ( !not ) {
         this.types |= typeBit;
@@ -1418,6 +1286,11 @@ FilterParser.prototype.parseOptions = function(s) {
         }
         if ( this.toNormalizedType.hasOwnProperty(opt) ) {
             this.parseOptType(opt, not);
+            // Due to ABP categorizing `websocket` requests as `other`, we need
+            // to add `websocket` for when `other` is used.
+            if ( opt === 'other' ) {
+                this.parseOptType('websocket', not);
+            }
             continue;
         }
         if ( opt.startsWith('domain=') ) {
@@ -1541,7 +1414,10 @@ FilterParser.prototype.parse = function(raw) {
         }
 
         // plain hostname? (from ABP filter list)
-        if ( this.reHostnameRule2.test(s) ) {
+        // https://github.com/gorhill/uBlock/issues/1757
+        // A filter can't be a pure-hostname one if there is a domain option
+        // present.
+        if ( this.domainOpt === '' && this.reHostnameRule2.test(s) ) {
             this.f = s.replace(this.reCleanupHostnameRule2, '');
             this.hostnamePure = true;
             return this;
@@ -1563,7 +1439,11 @@ FilterParser.prototype.parse = function(raw) {
     // normalize placeholders
     if ( this.reHasWildcard.test(s) ) {
         // remove pointless leading *
-        if ( s.startsWith('*') ) {
+        // https://github.com/gorhill/uBlock/issues/1669#issuecomment-224822448
+        // Keep the leading asterisk if we are dealing with a hostname-anchored
+        // filter, this will ensure the generic filter implementation is
+        // used.
+        if ( s.startsWith('*') && this.hostnameAnchored === false ) {
             s = s.replace(/^\*+([^%0-9a-z])/, '$1');
         }
         // remove pointless trailing *
@@ -1588,6 +1468,20 @@ FilterParser.prototype.parse = function(raw) {
 
     this.f = this.reHasUppercase.test(s) ? s.toLowerCase() : s;
 
+    // https://github.com/gorhill/uBlock/issues/1943#issuecomment-243188946
+    // Convert websocket-related filter where possible to a format which
+    // can be handled using CSP injection.
+    if (
+        this.cantWebsocket &&
+        this.anchor === -1 &&
+        this.firstParty === false &&
+        this.thirdParty === false &&
+        this.reWebsocketAny.test(this.f)
+    ) {
+        this.f = '*';
+        this.types = this.bitFromType('websocket');
+    }
+
     return this;
 };
 
@@ -1687,8 +1581,8 @@ FilterContainer.prototype.reset = function() {
     this.allowFilterCount = 0;
     this.blockFilterCount = 0;
     this.discardedCount = 0;
-    this.duplicateBuster = {};
-    this.categories = Object.create(null);
+    this.duplicateBuster = new Set();
+    this.categories = new Map();
     this.filterParser.reset();
     this.filterCounts = {};
 
@@ -1702,17 +1596,7 @@ FilterContainer.prototype.reset = function() {
 
 FilterContainer.prototype.freeze = function() {
     histogram('allFilters', this.categories);
-    this.duplicateBuster = {};
-
-    var categories = this.categories;
-    var bucket;
-    for ( var k in categories ) {
-        bucket = categories[k]['.'];
-        if ( bucket !== undefined ) {
-            bucket.freeze();
-        }
-    }
-
+    this.duplicateBuster = new Set();
     this.filterParser.reset();
     this.frozen = true;
 };
@@ -1745,20 +1629,23 @@ FilterContainer.prototype.factories = {
 /******************************************************************************/
 
 FilterContainer.prototype.toSelfie = function() {
-    var categoryToSelfie = function(dict) {
-        var selfie = [];
-        var bucket, ff, n, i, f;
-        for ( var token in dict ) {
-            // No need for hasOwnProperty() here: there is no prototype chain.
-            selfie.push('k2\t' + token);
-            bucket = dict[token];
+    var categoryToSelfie = function(map) {
+        var selfie = [],
+            iterator = map.entries(),
+            entry, bucket, ff, f;
+        for (;;) {
+            entry = iterator.next();
+            if ( entry.done ) {
+                break;
+            }
+            selfie.push('k2\t' + entry.value[0]);
+            bucket = entry.value[1];
             selfie.push(bucket.fid + '\t' + bucket.toSelfie());
             if ( bucket.fid !== '[]' ) {
                 continue;
             }
             ff = bucket.filters;
-            n = ff.length;
-            for ( i = 0; i < n; i++ ) {
+            for ( var i = 0, ni = ff.length; i < ni; i++ ) {
                 f = ff[i];
                 selfie.push(f.fid + '\t' + f.toSelfie());
             }
@@ -1766,12 +1653,17 @@ FilterContainer.prototype.toSelfie = function() {
         return selfie.join('\n');
     };
 
-    var categoriesToSelfie = function(dict) {
-        var selfie = [];
-        for ( var key in dict ) {
-            // No need for hasOwnProperty() here: there is no prototype chain.
-            selfie.push('k1\t' + key);
-            selfie.push(categoryToSelfie(dict[key]));
+    var categoriesToSelfie = function(map) {
+        var selfie = [],
+            iterator = map.entries(),
+            entry;
+        for (;;) {
+            entry = iterator.next();
+            if ( entry.done ) {
+                break;
+            }
+            selfie.push('k1\t' + entry.value[0]);
+            selfie.push(categoryToSelfie(entry.value[1]));
         }
         return selfie.join('\n');
     };
@@ -1799,7 +1691,7 @@ FilterContainer.prototype.fromSelfie = function(selfie) {
     this.discardedCount = selfie.discardedCount;
 
     var catKey, tokenKey;
-    var dict = this.categories, subdict;
+    var map = this.categories, submap;
     var bucket = null;
     var rawText = selfie.categories;
     var rawEnd = rawText.length;
@@ -1816,7 +1708,8 @@ FilterContainer.prototype.fromSelfie = function(selfie) {
         what = line.slice(0, pos);
         if ( what === 'k1' ) {
             catKey = line.slice(pos + 1);
-            subdict = dict[catKey] = Object.create(null);
+            submap = new Map();
+            map.set(catKey, submap);
             bucket = null;
             continue;
         }
@@ -1827,7 +1720,8 @@ FilterContainer.prototype.fromSelfie = function(selfie) {
         }
         factory = this.factories[what];
         if ( bucket === null ) {
-            bucket = subdict[tokenKey] = factory.fromSelfie(line.slice(pos + 1));
+            bucket = factory.fromSelfie(line.slice(pos + 1));
+            submap.set(tokenKey, bucket);
             continue;
         }
         // When token key is reused, it can't be anything
@@ -1941,9 +1835,11 @@ FilterContainer.prototype.compile = function(raw, out) {
 
 FilterContainer.prototype.compileHostnameOnlyFilter = function(parsed, out) {
     // Can't fit the filter in a pure hostname dictionary.
-    if ( parsed.domainOpt.length !== 0 ) {
-        return;
-    }
+    // https://github.com/gorhill/uBlock/issues/1757
+    // This should no longer happen with fix to above issue.
+    //if ( parsed.domainOpt.length !== 0 ) {
+    //    return;
+    //}
 
     var party = AnyParty;
     if ( parsed.firstParty !== parsed.thirdParty ) {
@@ -1982,10 +1878,6 @@ FilterContainer.prototype.compileHostnameOnlyFilter = function(parsed, out) {
 
 FilterContainer.prototype.compileFilter = function(parsed, out) {
     parsed.makeToken();
-    if ( parsed.token === '*' && parsed.hostnameAnchored ) {
-        console.error('FilterContainer.compileFilter("%s"): invalid filter', parsed.f);
-        return false;
-    }
 
     var party = AnyParty;
     if ( parsed.firstParty !== parsed.thirdParty ) {
@@ -2048,76 +1940,71 @@ FilterContainer.prototype.compileToAtomicFilter = function(filterClass, parsed,
 
 /******************************************************************************/
 
-FilterContainer.prototype.fromCompiledContent = function(text, lineBeg) {
-    var lineEnd;
-    var textEnd = text.length;
-    var line, fields, bucket, entry, factory, filter;
+FilterContainer.prototype.fromCompiledContent = function(lineIter) {
+    var line, hash, token, fclass, fdata,
+        bucket, entry, factory, filter,
+        fieldIter = new µb.FieldIterator('\v');
 
-    while ( lineBeg < textEnd ) {
-        if ( text.charCodeAt(lineBeg) !== 0x6E /* 'n' */ ) {
-            return lineBeg;
+    while ( lineIter.eot() === false ) {
+        if ( lineIter.text.charCodeAt(lineIter.offset) !== 0x6E /* 'n' */ ) {
+            return;
         }
-        lineEnd = text.indexOf('\n', lineBeg);
-        if ( lineEnd === -1 ) {
-            lineEnd = textEnd;
-        }
-        line = text.slice(lineBeg + 2, lineEnd);
-        fields = line.split('\v');
-        lineBeg = lineEnd + 1;
+        line = lineIter.next();
+
+        fieldIter.first(line);
+        hash = fieldIter.next();
+        token = fieldIter.next();
+        fclass = fieldIter.next();
+        fdata = fieldIter.next();
 
         // Special cases: delegate to more specialized engines.
         // Redirect engine.
-        if ( fields[2] === '=>' ) {
-            µb.redirectEngine.fromCompiledRule(fields[3]);
+        if ( fclass === '=>' ) {
+            µb.redirectEngine.fromCompiledRule(fdata);
             continue;
         }
 
         // Plain static filters.
         this.acceptedCount += 1;
 
-        bucket = this.categories[fields[0]];
+        bucket = this.categories.get(hash);
         if ( bucket === undefined ) {
-            bucket = this.categories[fields[0]] = Object.create(null);
+            bucket = new Map();
+            this.categories.set(hash, bucket);
         }
-        entry = bucket[fields[1]];
+        entry = bucket.get(token);
 
-        if ( fields[1] === '.' ) {
+        if ( token === '.' ) {
             if ( entry === undefined ) {
-                entry = bucket['.'] = new FilterHostnameDict();
+                entry = new FilterHostnameDict();
+                bucket.set('.', entry);
             }
-            if ( entry.add(fields[2]) === false ) {
+            // 'fclass' is hostname
+            if ( entry.add(fclass) === false ) {
                 this.discardedCount += 1;
             }
             continue;
         }
 
-        if ( this.duplicateBuster.hasOwnProperty(line) ) {
+        if ( this.duplicateBuster.has(line) ) {
             this.discardedCount += 1;
             continue;
         }
-        this.duplicateBuster[line] = true;
+        this.duplicateBuster.add(line);
 
-        factory = this.factories[fields[2]];
+        factory = this.factories[fclass];
 
-        // For development purpose
-        //if ( this.filterCounts.hasOwnProperty(fields[2]) === false ) {
-        //    this.filterCounts[fields[2]] = 1;
-        //} else {
-        //    this.filterCounts[fields[2]]++;
-        //}
-
-        filter = factory.fromSelfie(fields[3]);
+        filter = factory.fromSelfie(fdata);
         if ( entry === undefined ) {
-            bucket[fields[1]] = filter;
+            bucket.set(token, filter);
             continue;
         }
         if ( entry.fid === '[]' ) {
             entry.add(filter);
             continue;
         }
-        bucket[fields[1]] = new FilterBucket(entry, filter);
+        bucket.set(token, new FilterBucket(entry, filter));
     }
-    return textEnd;
 };
 
 /******************************************************************************/
@@ -2229,11 +2116,13 @@ FilterContainer.prototype.filterRegexFromCompiled = function(compiled, flags) {
     case '1ah':
     case '_':
     case '_h':
+        re = strToRegex(tfields[0], 0, flags);
+        break;
     case '||a':
     case '||ah':
     case '||_':
     case '||_h':
-        re = strToRegex(tfields[0], 0, flags);
+        re = strToRegex('||' + tfields[0], 0, flags);
         break;
     case '|a':
     case '|ah':
@@ -2256,9 +2145,12 @@ FilterContainer.prototype.filterRegexFromCompiled = function(compiled, flags) {
 
 /******************************************************************************/
 
+// bucket: Map
+// url: string
+
 FilterContainer.prototype.matchTokens = function(bucket, url) {
     // Hostname-only filters
-    var f = bucket['.'];
+    var f = bucket.get('.');
     if ( f !== undefined && f.match() ) {
         this.tokenRegister = '.';
         this.fRegister = f;
@@ -2274,7 +2166,7 @@ FilterContainer.prototype.matchTokens = function(bucket, url) {
         if ( token === '' ) {
             break;
         }
-        f = bucket[token];
+        f = bucket.get(token);
         if ( f !== undefined && f.match(url, tokenEntry.beg) ) {
             this.tokenRegister = token;
             this.fRegister = f;
@@ -2283,7 +2175,7 @@ FilterContainer.prototype.matchTokens = function(bucket, url) {
     }
 
     // Regex-based filters
-    f = bucket['*'];
+    f = bucket.get('*');
     if ( f !== undefined && f.match(url) ) {
         this.tokenRegister = '*';
         this.fRegister = f;
@@ -2303,8 +2195,8 @@ FilterContainer.prototype.matchTokens = function(bucket, url) {
 
 FilterContainer.prototype.matchStringExactType = function(context, requestURL, requestType) {
     // Be prepared to support unknown types
-    var type = typeNameToTypeValue[requestType] || 0;
-    if ( type === 0 ) {
+    var type = typeNameToTypeValue[requestType];
+    if ( type === undefined ) {
         return undefined;
     }
 
@@ -2328,7 +2220,7 @@ FilterContainer.prototype.matchStringExactType = function(context, requestURL, r
     if ( requestType === 'elemhide' ) {
         key = AllowAnyParty | type;
         if (
-            (bucket = categories[toHex(key)]) &&
+            (bucket = categories.get(toHex(key))) &&
             this.matchTokens(bucket, url)
         ) {
             this.keyRegister = key;
@@ -2340,14 +2232,14 @@ FilterContainer.prototype.matchStringExactType = function(context, requestURL, r
     // https://github.com/chrisaljoudi/uBlock/issues/139
     // Test against important block filters
     key = BlockAnyParty | Important | type;
-    if ( (bucket = categories[toHex(key)]) ) {
+    if ( (bucket = categories.get(toHex(key))) ) {
         if ( this.matchTokens(bucket, url) ) {
             this.keyRegister = key;
             return true;
         }
     }
     key = BlockAction | Important | type | party;
-    if ( (bucket = categories[toHex(key)]) ) {
+    if ( (bucket = categories.get(toHex(key))) ) {
         if ( this.matchTokens(bucket, url) ) {
             this.keyRegister = key;
             return true;
@@ -2356,14 +2248,14 @@ FilterContainer.prototype.matchStringExactType = function(context, requestURL, r
 
     // Test against block filters
     key = BlockAnyParty | type;
-    if ( (bucket = categories[toHex(key)]) ) {
+    if ( (bucket = categories.get(toHex(key))) ) {
         if ( this.matchTokens(bucket, url) ) {
             this.keyRegister = key;
         }
     }
     if ( this.fRegister === null ) {
         key = BlockAction | type | party;
-        if ( (bucket = categories[toHex(key)]) ) {
+        if ( (bucket = categories.get(toHex(key))) ) {
             if ( this.matchTokens(bucket, url) ) {
                 this.keyRegister = key;
             }
@@ -2377,14 +2269,14 @@ FilterContainer.prototype.matchStringExactType = function(context, requestURL, r
 
     // Test against allow filters
     key = AllowAnyParty | type;
-    if ( (bucket = categories[toHex(key)]) ) {
+    if ( (bucket = categories.get(toHex(key))) ) {
         if ( this.matchTokens(bucket, url) ) {
             this.keyRegister = key;
             return false;
         }
     }
     key = AllowAction | type | party;
-    if ( (bucket = categories[toHex(key)]) ) {
+    if ( (bucket = categories.get(toHex(key))) ) {
         if ( this.matchTokens(bucket, url) ) {
             this.keyRegister = key;
             return false;
@@ -2400,8 +2292,10 @@ FilterContainer.prototype.matchString = function(context) {
     // https://github.com/chrisaljoudi/uBlock/issues/519
     // Use exact type match for anything beyond `other`
     // Also, be prepared to support unknown types
-    var type = typeNameToTypeValue[context.requestType] || typeOtherValue;
-    if ( type > typeOtherValue ) {
+    var type = typeNameToTypeValue[context.requestType];
+    if ( type === undefined ) {
+         type = typeOtherValue;
+    } else if ( type === 0 || type > typeOtherValue ) {
         return this.matchStringExactType(context, context.requestURL, context.requestType);
     }
 
@@ -2445,28 +2339,28 @@ FilterContainer.prototype.matchString = function(context) {
     // evaluation. Normally, it is "evaluate block then evaluate allow", with
     // the `important` property it is "evaluate allow then evaluate block".
     key = BlockAnyTypeAnyParty | Important;
-    if ( (bucket = categories[toHex(key)]) ) {
+    if ( (bucket = categories.get(toHex(key))) ) {
         if ( this.matchTokens(bucket, url) ) {
             this.keyRegister = key;
             return true;
         }
     }
     key = BlockAnyType | Important | party;
-    if ( (bucket = categories[toHex(key)]) ) {
+    if ( (bucket = categories.get(toHex(key))) ) {
         if ( this.matchTokens(bucket, url) ) {
             this.keyRegister = key;
             return true;
         }
     }
     key = BlockAnyParty | Important | type;
-    if ( (bucket = categories[toHex(key)]) ) {
+    if ( (bucket = categories.get(toHex(key))) ) {
         if ( this.matchTokens(bucket, url) ) {
             this.keyRegister = key;
             return true;
         }
     }
     key = BlockAction | Important | type | party;
-    if ( (bucket = categories[toHex(key)]) ) {
+    if ( (bucket = categories.get(toHex(key))) ) {
         if ( this.matchTokens(bucket, url) ) {
             this.keyRegister = key;
             return true;
@@ -2475,28 +2369,28 @@ FilterContainer.prototype.matchString = function(context) {
 
     // Test against block filters
     key = BlockAnyTypeAnyParty;
-    if ( (bucket = categories[toHex(key)]) ) {
+    if ( (bucket = categories.get(toHex(key))) ) {
         if ( this.matchTokens(bucket, url) ) {
             this.keyRegister = key;
         }
     }
     if ( this.fRegister === null ) {
         key = BlockAnyType | party;
-        if ( (bucket = categories[toHex(key)]) ) {
+        if ( (bucket = categories.get(toHex(key))) ) {
             if ( this.matchTokens(bucket, url) ) {
                 this.keyRegister = key;
             }
         }
         if ( this.fRegister === null ) {
             key = BlockAnyParty | type;
-            if ( (bucket = categories[toHex(key)]) ) {
+            if ( (bucket = categories.get(toHex(key))) ) {
                 if ( this.matchTokens(bucket, url) ) {
                     this.keyRegister = key;
                 }
             }
             if ( this.fRegister === null ) {
                 key = BlockAction | type | party;
-                if ( (bucket = categories[toHex(key)]) ) {
+                if ( (bucket = categories.get(toHex(key))) ) {
                     if ( this.matchTokens(bucket, url) ) {
                         this.keyRegister = key;
                     }
@@ -2512,28 +2406,28 @@ FilterContainer.prototype.matchString = function(context) {
 
     // Test against allow filters
     key = AllowAnyTypeAnyParty;
-    if ( (bucket = categories[toHex(key)]) ) {
+    if ( (bucket = categories.get(toHex(key))) ) {
         if ( this.matchTokens(bucket, url) ) {
             this.keyRegister = key;
             return false;
         }
     }
     key = AllowAnyType | party;
-    if ( (bucket = categories[toHex(key)]) ) {
+    if ( (bucket = categories.get(toHex(key))) ) {
         if ( this.matchTokens(bucket, url) ) {
             this.keyRegister = key;
             return false;
         }
     }
     key = AllowAnyParty | type;
-    if ( (bucket = categories[toHex(key)]) ) {
+    if ( (bucket = categories.get(toHex(key))) ) {
         if ( this.matchTokens(bucket, url) ) {
             this.keyRegister = key;
             return false;
         }
     }
     key = AllowAction | type | party;
-    if ( (bucket = categories[toHex(key)]) ) {
+    if ( (bucket = categories.get(toHex(key))) ) {
         if ( this.matchTokens(bucket, url) ) {
             this.keyRegister = key;
             return false;
