@@ -200,6 +200,7 @@ housekeep itself.
         this.rootDomain = '';
         this.commitTimer = null;
         this.gcTimer = null;
+        this.onGCBarrier = false;
         this.netFiltering = true;
         this.netFilteringReadTime = 0;
 
@@ -226,11 +227,21 @@ housekeep itself.
     };
 
     TabContext.prototype.onGC = function() {
-        this.gcTimer = null;
         if ( vAPI.isBehindTheSceneTabId(this.tabId) ) {
             return;
         }
+        // https://github.com/gorhill/uBlock/issues/1713
+        // For unknown reasons, Firefox's setTimeout() will sometimes
+        // causes the callback function to be called immediately, bypassing
+        // the main event loop. For now this should prevent uBO from crashing
+        // as a result of the bad setTimeout() behavior.
+        if ( this.onGCBarrier ) {
+            return;
+        }
+        this.onGCBarrier = true;
+        this.gcTimer = null;
         vAPI.tabs.get(this.tabId, this.onTab.bind(this));
+        this.onGCBarrier = false;
     };
 
     // https://github.com/gorhill/uBlock/issues/248
@@ -380,12 +391,15 @@ housekeep itself.
         return tabContexts[vAPI.noTabId];
     };
 
+    // https://github.com/gorhill/uBlock/issues/1735
+    //   Filter for popups if actually committing.
     var commit = function(tabId, url) {
         var entry = tabContexts[tabId];
         if ( entry === undefined ) {
             entry = push(tabId, url);
         } else {
             entry.commit(url);
+            popupCandidateTest(tabId);
         }
         return entry;
     };
@@ -405,7 +419,11 @@ housekeep itself.
     })();
 
     // Context object, typically to be used to feed filtering engines.
+    var contextJunkyard = [];
     var Context = function(tabId) {
+        this.init(tabId);
+    };
+    Context.prototype.init = function(tabId) {
         var tabContext = lookup(tabId);
         this.rootHostname = tabContext.rootHostname;
         this.rootDomain = tabContext.rootDomain;
@@ -414,9 +432,16 @@ housekeep itself.
         this.requestURL =
         this.requestHostname =
         this.requestDomain = '';
+        return this;
+    };
+    Context.prototype.dispose = function() {
+        contextJunkyard.push(this);
     };
 
     var createContext = function(tabId) {
+        if ( contextJunkyard.length ) {
+            return contextJunkyard.pop().init(tabId);
+        }
         return new Context(tabId);
     };
 
@@ -512,8 +537,6 @@ vAPI.tabs.onClosed = function(tabId) {
 // d: close target
 
 vAPI.tabs.onPopupUpdated = (function() {
-    //console.debug('vAPI.tabs.onPopup: details = %o', details);
-
     // The same context object will be reused everytime. This also allows to
     // remember whether a popup or popunder was matched.
     var context = {};
@@ -553,9 +576,15 @@ vAPI.tabs.onPopupUpdated = (function() {
         context.requestType = 'popup';
 
         // https://github.com/gorhill/uBlock/commit/1d448b85b2931412508aa01bf899e0b6f0033626#commitcomment-14944764
-        // Ignore bad target URL. On Firefox, an `about:blank` tab may be
-        // opened for a new tab before it is filled in with the real target URL.
-        if ( context.requestHostname === '' ) {
+        //   Ignore bad target URL. On Firefox, an `about:blank` tab may be
+        //   opened for a new tab before it is filled in with the real target
+        //   URL.
+        // https://github.com/gorhill/uBlock/issues/1735
+        //   Do not bail out on `data:` URI, they are commonly used for popups.
+        if (
+            context.requestHostname === '' &&
+            targetURL.startsWith('data:') === false
+        ) {
             return '';
         }
 
@@ -605,22 +634,7 @@ vAPI.tabs.onPopupUpdated = (function() {
         return '';
     };
 
-    var popunderMatch = function(openerURL, targetURL) {
-        var result = popupMatch(targetURL, openerURL, null, 'popunder');
-        if ( µb.isBlockResult(result) ) {
-            return result;
-        }
-        // https://github.com/gorhill/uBlock/issues/1010#issuecomment-186824878
-        //   Check the opener tab as if it were the newly opened tab: if there
-        //   is a hit against a popup filter, and if the matching filter is not
-        //   a broad one, we will consider the opener tab to be a popunder tab.
-        //   For now, a "broad" filter is one which does not touch any part of
-        //   the hostname part of the opener URL.
-        var openerHostname = µb.URI.hostnameFromURI(openerURL);
-        if ( openerHostname === '' ) {
-            return '';
-        }
-        result = popupMatch(targetURL, openerURL, null, 'popup');
+    var mapPopunderResult = function(popunderURL, popunderHostname, result) {
         if ( result.startsWith('sb:') === false ) {
             return '';
         }
@@ -637,23 +651,64 @@ vAPI.tabs.onPopupUpdated = (function() {
         if ( re === null ) {
             return '';
         }
-        var matches = re.exec(openerURL);
+        var matches = re.exec(popunderURL);
         if ( matches === null ) {
             return '';
         }
         var beg = matches.index,
             end = beg + matches[0].length,
-            pos = openerURL.indexOf(openerHostname);
+            pos = popunderURL.indexOf(popunderHostname);
         if ( pos === -1 ) {
             return '';
         }
         // https://github.com/gorhill/uBlock/issues/1471
-        // We test whether the opener hostname as at least one character
-        // within matched portion of URL.
-        if ( beg >= pos + openerHostname.length || end <= pos ) {
+        //   We test whether the opener hostname as at least one character
+        //   within matched portion of URL.
+        // https://github.com/gorhill/uBlock/issues/1903
+        //   Ignore filters which cause a match before the start of the
+        //   hostname in the URL.
+        return beg >= pos &&
+               beg < pos + popunderHostname.length &&
+               end > pos ?
+           result :
+           '';
+    };
+
+    var popunderMatch = function(openerURL, targetURL) {
+        var result = popupMatch(targetURL, openerURL, null, 'popunder');
+        if ( µb.isBlockResult(result) ) {
+            return result;
+        }
+        // https://github.com/gorhill/uBlock/issues/1010#issuecomment-186824878
+        //   Check the opener tab as if it were the newly opened tab: if there
+        //   is a hit against a popup filter, and if the matching filter is not
+        //   a broad one, we will consider the opener tab to be a popunder tab.
+        //   For now, a "broad" filter is one which does not touch any part of
+        //   the hostname part of the opener URL.
+        var popunderURL = openerURL;
+        var popunderHostname = µb.URI.hostnameFromURI(popunderURL);
+        if ( popunderHostname === '' ) {
             return '';
         }
-        return result;
+        result = mapPopunderResult(
+            popunderURL,
+            popunderHostname,
+            popupMatch(targetURL, popunderURL, null, 'popup')
+        );
+        if ( result !== '' ) {
+            return result;
+        }
+        // https://github.com/gorhill/uBlock/issues/1598
+        // Try to find a match against origin part of the opener URL.
+        popunderURL = µb.URI.originFromURI(popunderURL);
+        if ( popunderURL === '' ) {
+            return '';
+        }
+        return mapPopunderResult(
+            popunderURL,
+            popunderHostname,
+            popupMatch(targetURL, popunderURL, null, 'popup')
+        );
     };
 
     return function(targetTabId, openerTabId) {
@@ -675,6 +730,11 @@ vAPI.tabs.onPopupUpdated = (function() {
             return;
         }
 
+        // https://github.com/gorhill/uBlock/issues/1538
+        if ( µb.getNetFilteringSwitch(µb.normalizePageURL(openerTabId, openerURL)) === false ) {
+            return;
+        }
+
         // If the page URL is that of our "blocked page" URL, extract the URL of
         // the page which was blocked.
         if ( targetURL.startsWith(vAPI.getURL('document-blocked.html')) ) {
