@@ -8,8 +8,8 @@ DES=dist/build/uBlock0.firefox
 rm -rf $DES
 mkdir -p $DES
 
-cp -R assets                            $DES/
-rm    $DES/assets/*.sh
+bash ./tools/make-assets.sh $DES
+
 cp -R src/css                           $DES/
 cp -R src/img                           $DES/
 cp -R src/js                            $DES/
@@ -22,6 +22,7 @@ cat   src/background.html | sed -e '/vapi-polyfill\.js/d' > $DES/background.html
 
 mv    $DES/img/icon_128.png             $DES/icon.png
 cp    platform/firefox/css/*            $DES/css/
+cp    platform/firefox/polyfill.js      $DES/js/
 cp    platform/firefox/vapi-*.js        $DES/js/
 cp    platform/firefox/bootstrap.js     $DES/
 cp    platform/firefox/frame*.js        $DES/
@@ -32,13 +33,14 @@ cp    platform/firefox/*.xul            $DES/
 cp    LICENSE.txt                       $DES/
 
 echo "*** uBlock0.firefox: Generating meta..."
-python tools/make-firefox-meta.py $DES/ "$2"
+python tools/make-firefox-meta.py $DES/
 
 if [ "$1" = all ]; then
+    set +v
     echo "*** uBlock0.firefox: Creating package..."
-    pushd $DES/
+    pushd $DES/ > /dev/null
     zip ../uBlock0.firefox.xpi -qr *
-    popd
+    popd > /dev/null
 fi
 
 echo "*** uBlock0.firefox: Package done."
