@@ -9,8 +9,8 @@ DES=dist/build/uBlock0.opera
 rm -r $DES
 mkdir -p $DES
 
-cp -R assets $DES/
-rm $DES/assets/*.sh
+bash ./tools/make-assets.sh $DES
+
 cp -R src/css $DES/
 cp -R src/img $DES/
 cp -R src/js $DES/
